@@ -1,19 +1,19 @@
-########
-#
-# compare_batch_results.py
-# 
-# Compare sets of batch results; typically used to compare:
-#
-# * MegaDetector versions
-# * Results before/after RDE
-# * Results with/without augmentation
-#
-# Makes pairwise comparisons, but can take lists of results files (will perform 
-# all pairwise comparisons).  Results are written to an HTML page that shows the number
-# and nature of disagreements (in the sense of each image being a detection or non-detection), 
-# with sample images for each category.
-# 
-########
+"""
+
+compare_batch_results.py
+
+Compare sets of batch results; typically used to compare:
+
+* Results from different MegaDetector versions
+* Results before/after RDE
+* Results with/without augmentation
+
+Makes pairwise comparisons, but can take lists of results files (will perform 
+all pairwise comparisons).  Results are written to an HTML page that shows the number
+and nature of disagreements (in the sense of each image being a detection or non-detection), 
+with sample images for each category.
+
+"""
 
 #%% Imports
 
@@ -36,9 +36,6 @@ from md_utils import path_utils
 
 
 #%% Constants and support classes
-
-# We will confirm that this matches what we load from each file
-default_detection_categories = {'1': 'animal', '2': 'person', '3': 'vehicle'}
     
 class PairwiseBatchComparisonOptions:
     """
@@ -46,16 +43,28 @@ class PairwiseBatchComparisonOptions:
     pairwise options sets is stored in the BatchComparisonsOptions class.
     """
     
+    #: First filename to compare
     results_filename_a = None
+    
+    #: Second filename to compare
     results_filename_b = None
     
+    #: Description to use in the output HTML for filename A
     results_description_a = None
+    
+    #: Description to use in the output HTML for filename B
     results_description_b = None
     
-    detection_thresholds_a = {'animal':0.15,'person':0.15,'vehicle':0.15}
-    detection_thresholds_b = {'animal':0.15,'person':0.15,'vehicle':0.15}
+    #: Per-class detection thresholds to use for filename A (including a 'default' threshold)
+    detection_thresholds_a = {'animal':0.15,'person':0.15,'vehicle':0.15,'default':0.15}
+    
+    #: Per-class detection thresholds to use for filename B (including a 'default' threshold)
+    detection_thresholds_b = {'animal':0.15,'person':0.15,'vehicle':0.15,'default':0.15}
 
+    #: Rendering threshold to use for all categories for filename A
     rendering_confidence_threshold_a = 0.1
+    
+    #: Rendering threshold to use for all categories for filename B
     rendering_confidence_threshold_b = 0.1
 
 # ...class PairwiseBatchComparisonOptions
@@ -66,23 +75,56 @@ class BatchComparisonOptions:
     Defines the options for a set of (possibly many) pairwise comparisons.
     """
     
+    #: Folder to which we should write HTML output
     output_folder = None
+    
+    #: Base folder for images (which are specified as relative files)
     image_folder = None
+    
+    #: Job name to use in the HTML output file
     job_name = ''
     
+    #: Maximum number of images to render for each category, where a "category" here is
+    #: "detections_a_only", "detections_b_only", etc., or None to render all images.
     max_images_per_category = 1000
+    
+    #: Maximum number of images per HTML page (paginates if a category page goes beyond this),
+    #: or None to disable pagination.
+    max_images_per_page = None
+    
+    #: Colormap to use for detections in file A (maps detection categories to colors)
     colormap_a = ['Red']
+    
+    #: Colormap to use for detections in file B (maps detection categories to colors)
     colormap_b = ['RoyalBlue']
 
-    # Process-based parallelization isn't supported yet; this must be "True"
+    #: Process-based parallelization isn't supported yet; this must be "True"
     parallelize_rendering_with_threads = True
     
+    #: List of filenames to include in the comparison, or None to use all files
+    filenames_to_include = None
+    
+    #: Compare only detections/non-detections, ignore categories (still renders categories)
+    class_agnostic_comparison = False
+    
+    #: Width of images to render in the output HTML
     target_width = 800
+    
+    #: Number of workers to use for rendering, or <=1 to disable parallelization
     n_rendering_workers = 20
+    
+    #: Random seed for image sampling (not used if max_images_per_category is None)
     random_seed = 0
     
+    #: Whether to sort results by confidence; if this is False, sorts by filename
+    sort_by_confidence = False
+    
+    #: The expectation is that all results sets being compared will refer to the same images; if this
+    #: is True (default), we'll error if that's not the case, otherwise non-matching lists will just be
+    #: a warning.
     error_on_non_matching_lists = True
     
+    #: List of PairwiseBatchComparisonOptions that defines the comparisons we'll render.
     pairwise_options = []
     
 # ...class BatchComparisonOptions
@@ -90,21 +132,24 @@ class BatchComparisonOptions:
 
 class PairwiseBatchComparisonResults:
     """
-    The results from a single pairwise comparison
+    The results from a single pairwise comparison.
     """
     
+    #: String of HTML content suitable for rendering to an HTML file
     html_content = None
+    
+    #: Possibly-modified version of the PairwiseBatchComparisonOptions supplied as input.
     pairwise_options = None
     
-    # A dictionary with keys including:
+    #: A dictionary with keys including:
+    #:
+    #: common_detections
+    #: common_non_detections
+    #: detections_a_only
+    #: detections_b_only
+    #: class_transitions
     #
-    # "common_detections"
-    # common_non_detections
-    # detections_a_only
-    # detections_b_only
-    # class_transitions
-    #
-    # Each of these maps a filename to a two-element list (the image in set A, the image in set B).
+    #: Each of these maps a filename to a two-element list (the image in set A, the image in set B).
     categories_to_image_pairs = None
 
 # ...class PairwiseBatchComparisonResults
@@ -115,9 +160,10 @@ class BatchComparisonResults:
     The results from a set of pairwise comparisons
     """
     
+    #: Filename containing HTML output
     html_output_file = None
     
-    # An list of PairwiseBatchComparisonResults
+    #: A list of PairwiseBatchComparisonResults
     pairwise_results = None
     
 # ...class BatchComparisonResults    
@@ -137,9 +183,20 @@ main_page_footer = '<br/><br/><br/></body></html>\n'
 
 #%% Comparison functions
 
-def render_image_pair(fn,image_pairs,category_folder,options,pairwise_options):
+def _render_image_pair(fn,image_pairs,category_folder,options,pairwise_options):
     """
     Render two sets of results (i.e., a comparison) for a single image.
+    
+    Args:
+        fn (str): image filename
+        image_pairs (dict): dict mapping filenames to pairs of image dicts
+        category_folder (str): folder to which to render this image, typically 
+            "detections_a_only", "detections_b_only", etc.
+        options (BatchComparisonOptions): job options
+        pairwise_options (PairwiseBatchComparisonOptions): pairwise comparison options
+            
+    Returns:
+        str: rendered image filename            
     """
     
     input_image_path = os.path.join(options.image_folder,fn)
@@ -187,27 +244,30 @@ def render_image_pair(fn,image_pairs,category_folder,options,pairwise_options):
     im.save(output_image_path)
     return output_image_path
 
-# ...def render_image_pair()
+# ...def _render_image_pair()
 
 
-def pairwise_compare_batch_results(options,output_index,pairwise_options):
+def _pairwise_compare_batch_results(options,output_index,pairwise_options):
     """
     The main entry point for this module is compare_batch_results(), which calls 
     this function for each pair of comparisons the caller has requested.  Generates an
     HTML page for this comparison.  Returns a BatchComparisonResults object.
     
-    options: an instance of BatchComparisonOptions
-    
-    output_index: a numeric index used for generating HTML titles
-    
-    pairwise_options: an instance of PairwiseBatchComparisonOptions    
+    Args:
+        options (BatchComparisonOptions): overall job options for this comparison group
+        output_index (int): a numeric index used for generating HTML titles    
+        pairwise_options (PairwiseBatchComparisonOptions): job options for this comparison
+        
+    Returns:
+        PairwiseBatchComparisonResults: the results of this pairwise comparison
     """
     
     # pairwise_options is passed as a parameter here, and should not be specified
     # in the options object.
     assert options.pairwise_options is None
     
-    random.seed(options.random_seed)
+    if options.random_seed is not None:
+        random.seed(options.random_seed)
 
     # Warn the user if some "detections" might not get rendered
     max_classification_threshold_a = max(list(pairwise_options.detection_thresholds_a.values()))
@@ -241,10 +301,20 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
     with open(pairwise_options.results_filename_b,'r') as f:
         results_b = json.load(f)
         
-    # assert results_a['detection_categories'] == default_detection_categories
-    # assert results_b['detection_categories'] == default_detection_categories
-    assert results_a['detection_categories'] == results_b['detection_categories']
-    detection_categories = results_a['detection_categories']
+    # Don't let path separators confuse things
+    for im in results_a['images']:
+        if 'file' in im:
+            im['file'] = im['file'].replace('\\','/')
+    for im in results_b['images']:
+        if 'file' in im:
+            im['file'] = im['file'].replace('\\','/')
+    
+    if not options.class_agnostic_comparison:
+        assert results_a['detection_categories'] == results_b['detection_categories'], \
+            "Cannot perform a class-sensitive comparison across results with different categories"
+            
+    detection_categories_a = results_a['detection_categories']
+    detection_categories_b = results_b['detection_categories']
     
     if pairwise_options.results_description_a is None:
         if 'detector' not in results_a['info']:
@@ -273,7 +343,7 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
     filenames_b_set = set([im['file'] for im in images_b])
     
     if len(images_a) != len(images_b):
-        s = 'set A has {} iamges, set B has {}'.format(len(images_a),len(images_b))
+        s = 'set A has {} images, set B has {}'.format(len(images_a),len(images_b))
         if options.error_on_non_matching_lists:
             raise ValueError(s)
         else:
@@ -286,6 +356,10 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
     assert len(filenames_a) == len(images_a)
     assert len(filenames_b_set) == len(images_b)    
     
+    if options.filenames_to_include is None:
+        filenames_to_compare = filenames_a
+    else:
+        filenames_to_compare = options.filenames_to_include
     
     ##%% Find differences
     
@@ -298,9 +372,9 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
     detections_a_only = {}
     detections_b_only = {}
     class_transitions = {}
-    
-    # fn = filenames_a[0]
-    for fn in tqdm(filenames_a):
+        
+    # fn = filenames_to_compare[0]
+    for fn in tqdm(filenames_to_compare):
     
         if fn not in filename_to_image_b:
             
@@ -330,14 +404,19 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
             
             category_id = det['category']
             
-            if category_id not in detection_categories:
+            if category_id not in detection_categories_a:
                 print('Warning: unexpected category {} for model A on file {}'.format(category_id,fn))
                 invalid_category_error = True
                 break
                 
             conf = det['conf']
             
-            if conf >= pairwise_options.detection_thresholds_a[detection_categories[category_id]]:
+            if detection_categories_a[category_id] in pairwise_options.detection_thresholds_a:
+                conf_thresh = pairwise_options.detection_thresholds_a[detection_categories_a[category_id]]
+            else:
+                conf_thresh = pairwise_options.detection_thresholds_a['default']
+                
+            if conf >= conf_thresh:
                 categories_above_threshold_a.add(category_id)
                             
         if invalid_category_error:
@@ -349,14 +428,19 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
             
             category_id = det['category']
             
-            if category_id not in detection_categories:
+            if category_id not in detection_categories_b:
                 print('Warning: unexpected category {} for model B on file {}'.format(category_id,fn))
                 invalid_category_error = True
                 break
             
             conf = det['conf']
             
-            if conf >= pairwise_options.detection_thresholds_b[detection_categories[category_id]]:
+            if detection_categories_b[category_id] in pairwise_options.detection_thresholds_b:
+                conf_thresh = pairwise_options.detection_thresholds_b[detection_categories_b[category_id]]
+            else:
+                conf_thresh = pairwise_options.detection_thresholds_a['default']
+                
+            if conf >= conf_thresh:
                 categories_above_threshold_b.add(category_id)
                             
         if invalid_category_error:
@@ -368,7 +452,8 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
         detection_b = (len(categories_above_threshold_b) > 0)
                 
         if detection_a and detection_b:            
-            if categories_above_threshold_a == categories_above_threshold_b:
+            if (categories_above_threshold_a == categories_above_threshold_b) or \
+                options.class_agnostic_comparison:
                 common_detections[fn] = im_pair
             else:
                 class_transitions[fn] = im_pair
@@ -383,7 +468,7 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
     # ...for each filename
     
     print('Of {} files:\n{} common detections\n{} common non-detections\n{} A only\n{} B only\n{} class transitions'.format(
-        len(filenames_a),len(common_detections),
+        len(filenames_to_compare),len(common_detections),
         len(common_non_detections),len(detections_a_only),
         len(detections_b_only),len(class_transitions)))
         
@@ -430,11 +515,11 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
         if options.n_rendering_workers <= 1:
             output_image_paths = []
             for fn in tqdm(image_filenames):        
-                output_image_paths.append(render_image_pair(fn,image_pairs,category_folder,
+                output_image_paths.append(_render_image_pair(fn,image_pairs,category_folder,
                                                             options,pairwise_options))
         else:            
             output_image_paths = list(tqdm(pool.imap(
-                partial(render_image_pair, image_pairs=image_pairs, 
+                partial(_render_image_pair, image_pairs=image_pairs, 
                         category_folder=category_folder,options=options,
                         pairwise_options=pairwise_options),
                 image_filenames), 
@@ -453,14 +538,16 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
         # Choose detection pairs we're going to render for this category
         image_pairs = categories_to_image_pairs[category]
         image_filenames = list(image_pairs.keys())
-        if len(image_filenames) > options.max_images_per_category:
-            print('Sampling {} of {} image pairs for category {}'.format(
-                options.max_images_per_category,
-                len(image_filenames),
-                category))
-            image_filenames = random.sample(image_filenames,
-                                            options.max_images_per_category)
-        assert len(image_filenames) <= options.max_images_per_category
+        
+        if options.max_images_per_category is not None and options.max_images_per_category > 0:
+            if len(image_filenames) > options.max_images_per_category:
+                print('Sampling {} of {} image pairs for category {}'.format(
+                    options.max_images_per_category,
+                    len(image_filenames),
+                    category))
+                image_filenames = random.sample(image_filenames,
+                                                options.max_images_per_category)
+            assert len(image_filenames) <= options.max_images_per_category
 
         input_image_absolute_paths = [os.path.join(options.image_folder,fn) for fn in image_filenames]
         
@@ -492,15 +579,34 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
             max_conf_b = maxempty([det['conf'] for det in image_b['detections']])
             
             title = input_path_relative + ' (max conf {:.2f},{:.2f})'.format(max_conf_a,max_conf_b)
+            
+            # Only used if sort_by_confidence is True
+            if category == 'common_detections':
+                sort_conf = max(max_conf_a,max_conf_b)
+            elif category == 'common_non_detections':
+                sort_conf = max(max_conf_a,max_conf_b)
+            elif category == 'detections_a_only':
+                sort_conf = max_conf_a
+            elif category == 'detections_b_only':
+                sort_conf = max_conf_b
+            elif category == 'class_transitions':
+                sort_conf = max(max_conf_a,max_conf_b)
+            else:
+                print('Warning: unknown sort category {}'.format(category))
+                sort_conf = max(max_conf_a,max_conf_b)
+                
             info = {
                 'filename': fn,
                 'title': title,
                 'textStyle': 'font-family:verdana,arial,calibri;font-size:' + \
                     '80%;text-align:left;margin-top:20;margin-bottom:5',
-                'linkTarget': urllib.parse.quote(input_image_absolute_paths[i_fn])
+                'linkTarget': urllib.parse.quote(input_image_absolute_paths[i_fn]),
+                'sort_conf':sort_conf
             }
             image_info.append(info)
     
+        # ...for each image
+        
         category_page_header_string = '<h1>{}</h1>'.format(categories_to_page_titles[category])
         category_page_header_string += '<p style="font-weight:bold;">\n'
         category_page_header_string += 'Model A: {}<br/>\n'.format(
@@ -521,11 +627,18 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
             str(pairwise_options.rendering_confidence_threshold_b))
         category_page_header_string += '</p>\n'        
         
+        # Default to sorting by filename
+        if options.sort_by_confidence:
+            image_info = sorted(image_info, key=lambda d: d['sort_conf'], reverse=True)
+        else:
+            image_info = sorted(image_info, key=lambda d: d['filename'])
+            
         write_html_image_list(
             category_html_filename,
             images=image_info,
             options={
-                'headerHtml': category_page_header_string
+                'headerHtml': category_page_header_string,
+                'maxFiguresPerHtmlFile': options.max_images_per_page
             })
         
     # ...for each category
@@ -559,7 +672,7 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
     html_output_string += '<br/>'
     
     html_output_string += ('Of {} total files:<br/><br/><div style="margin-left:15px;">{} common detections<br/>{} common non-detections<br/>{} A only<br/>{} B only<br/>{} class transitions</div><br/>'.format(
-        len(filenames_a),len(common_detections),
+        len(filenames_to_compare),len(common_detections),
         len(common_non_detections),len(detections_a_only),
         len(detections_b_only),len(class_transitions)))
     
@@ -583,14 +696,20 @@ def pairwise_compare_batch_results(options,output_index,pairwise_options):
             
     return pairwise_results
         
-# ...def compare_batch_results()
+# ...def _pairwise_compare_batch_results()
 
 
 def compare_batch_results(options):
     """
     The main entry point for this module.  Runs one or more batch results comparisons, 
-    writing results to an html page.  Most of the work is deferred to
-    pairwise_compare_batch_results().
+    writing results to an html page.  Most of the work is deferred to _pairwise_compare_batch_results().
+    
+    Args:
+        options (BatchComparisonOptions): job options to use for this comparison task, including the
+            list of specific pairswise comparisons to make (in the pairwise_options field)
+            
+    Returns:
+        BatchComparisonResults: the results of this comparison task
     """
     
     assert options.output_folder is not None
@@ -614,7 +733,7 @@ def compare_batch_results(options):
     for i_comparison,pairwise_options in enumerate(pairwise_options_list):
         print('Running comparison {} of {}'.format(i_comparison,n_comparisons))
         pairwise_results = \
-            pairwise_compare_batch_results(options,i_comparison,pairwise_options)
+            _pairwise_compare_batch_results(options,i_comparison,pairwise_options)
         html_content += pairwise_results.html_content
         all_pairwise_results.append(pairwise_results)
 
@@ -641,6 +760,18 @@ def n_way_comparison(filenames,options,detection_thresholds=None,rendering_thres
     """
     Performs N pairwise comparisons for the list of results files in [filenames], by generating
     sets of pairwise options and calling compare_batch_results.
+    
+    Args:
+        filenames (list): list of MD results filenames to compare
+        options (BatchComparisonOptions): task options set in which pairwise_options is still 
+            empty; that will get populated from [filenames]
+        detection_thresholds (list, optional): list of detection thresholds with the same length
+            as [filenames], or None to use sensible defaults
+        rendering_thresholds (list, optional): list of rendering thresholds with the same length
+            as [filenames], or None to use sensible defaults
+    
+    Returns:
+        BatchComparisonResults: the results of this comparison task
     """
     
     if detection_thresholds is None:
@@ -663,12 +794,9 @@ def n_way_comparison(filenames,options,detection_thresholds=None,rendering_thres
         pairwise_options.rendering_confidence_threshold_a = rendering_thresholds[i]
         pairwise_options.rendering_confidence_threshold_b = rendering_thresholds[j]
         
-        pairwise_options.detection_thresholds_a = {'animal':detection_thresholds[i],
-                                                   'person':detection_thresholds[i],
-                                                   'vehicle':detection_thresholds[i]}
-        pairwise_options.detection_thresholds_b = {'animal':detection_thresholds[j],
-                                                   'person':detection_thresholds[j],
-                                                   'vehicle':detection_thresholds[j]}
+        pairwise_options.detection_thresholds_a = {'default':detection_thresholds[i]}
+        pairwise_options.detection_thresholds_b = {'default':detection_thresholds[j]}
+        
         options.pairwise_options.append(pairwise_options)
 
     return compare_batch_results(options)
@@ -679,32 +807,25 @@ def n_way_comparison(filenames,options,detection_thresholds=None,rendering_thres
 #%% Interactive driver
 
 if False:
-    
-    #%% Running KGA test
-    
-    # CUDA_VISIBLE_DEVICES=0 python run_detector_batch.py  ~/models/camera_traps/megadetector/md_v5.0.0/md_v5a.0.0.pt ~/data/KGA/ ~/data/KGA-5a.json --recursive --output_relative_filenames --quiet
-    # CUDA_VISIBLE_DEVICES=1 python run_detector_batch.py  ~/models/camera_traps/megadetector/md_v5.0.0/md_v5b.0.0.pt ~/data/KGA/ ~/data/KGA-5b.json --recursive --output_relative_filenames --quiet
-    
-    # python run_detector_batch.py  ~/models/camera_traps/megadetector/md_v4.1.0/md_v4.1.0.pb ~/data/KGA ~/data/KGA-4.json --recursive --output_relative_filenames --quiet
-    
-    # CUDA_VISIBLE_DEVICES=0 python run_detector_batch.py  ~/models/camera_traps/megadetector/md_v5.0.0/md_v5a.0.0.pt ~/data/KGA/ ~/data/KGA-5a-pillow-9.2.0.json --recursive --output_relative_filenames --quiet
-
-
+        
     #%% Test two-way comparison
     
     options = BatchComparisonOptions()
 
-    options.parallelize_rendering_with_threads = False
+    options.parallelize_rendering_with_threads = True
+    
+    options.job_name = 'BCT'
+    options.output_folder = r'g:\temp\comparisons'
+    options.image_folder = r'g:\camera_traps\camera_trap_images'
+    options.max_images_per_category = 100
+    options.sort_by_confidence = True
     
-    options.job_name = 'KGA-test'
-    options.output_folder = os.path.expanduser('~/tmp/md-comparison-test')
-    options.image_folder = os.path.expanduser('~/data/KGA')
-
     options.pairwise_options = []
 
-    filenames = [
-        os.path.expanduser('~/data/KGA-5a.json'),
-        os.path.expanduser('~/data/KGA-5b.json')
+    results_base = os.path.expanduser('~/postprocessing/bellevue-camera-traps')
+    filenames = [                
+        os.path.join(results_base,r'bellevue-camera-traps-2023-12-05-v5a.0.0\combined_api_outputs\bellevue-camera-traps-2023-12-05-v5a.0.0_detections.json'),
+        os.path.join(results_base,r'bellevue-camera-traps-2023-12-05-aug-v5a.0.0\combined_api_outputs\bellevue-camera-traps-2023-12-05-aug-v5a.0.0_detections.json')
         ]
 
     detection_thresholds = [0.15,0.15]
@@ -835,4 +956,3 @@ def main():
 if __name__ == '__main__':
     
     main()
-    
\ No newline at end of file
