@@ -1,16 +1,16 @@
-########
-#
-# cct_field_adjustments.py
-#
-# CCT metadata was posted with int locations instead of strings.
-#
-# This script fixes those issues and rev's the version number.
-#
-########
+"""
+
+ cct_field_adjustments.py
+
+ CCT metadata was posted with int locations instead of strings.
+
+ This script fixes those issues and rev's the version number.
+
+"""
 
 #%% Constants and environment
 
-from data_management.databases import sanity_check_json_db
+from data_management.databases import integrity_check_json_db
 import json
 import os
 
@@ -54,5 +54,4 @@ print('Finished writing output .json to {}'.format(outputJsonFile))
 
 #%% Check output data file
 
-sanity_check_json_db.sanity_check_json_db(outputJsonFile)
-
+integrity_check_json_db.integrity_check_json_db(outputJsonFile)
