@@ -1,93 +1,135 @@
-########
-#
-# visualize_db.py
-# 
-# Outputs an HTML page visualizing annotations (class labels and/or bounding boxes)
-# on a sample of images in a database in the COCO Camera Traps format
-#
-########
+"""
+
+visualize_db.py
+
+Outputs an HTML page visualizing annotations (class labels and/or bounding boxes)
+on a sample of images in a database in the COCO Camera Traps format.
+
+"""
 
 #%% Imports
 
 import argparse
 import inspect
+import random
 import json
 import math
 import os
 import sys
 import time
+
+import pandas as pd
+import numpy as np
+
+import humanfriendly
+
 from itertools import compress
 from multiprocessing.pool import ThreadPool
 from multiprocessing.pool import Pool
-
-import pandas as pd
 from tqdm import tqdm
-import humanfriendly
 
 from md_utils.write_html_image_list import write_html_image_list
+from data_management.cct_json_utils import IndexedJsonDb
 
 import md_visualization.visualization_utils as vis_utils
-from data_management.cct_json_utils import IndexedJsonDb
 
 
 #%% Settings
 
 class DbVizOptions:
+    """
+    Parameters controlling the behavior of visualize_db().
+    """
     
-    # Set to None to visualize all images
+    #: Number of images to sample from the database, or None to visualize all images
     num_to_visualize = None
     
-    # Target size for rendering; set either dimension to -1 to preserve aspect ratio
-    #
-    # If viz_size is None or (-1,-1), the original image size is used.
-    viz_size = (675, -1)
+    #: Target size for rendering; set either dimension to -1 to preserve aspect ratio.
+    #:
+    #: If viz_size is None or (-1,-1), the original image size is used.
+    viz_size = (800, -1)
+    
+    #: HTML rendering options; see write_html_image_list for details
+    #:
+    #:The most relevant option one might want to set here is:
+    #:
+    #: htmlOptions['maxFiguresPerHtmlFile']
+    #:
+    #: ...which can be used to paginate previews to a number of images that will load well
+    #: in a browser (5000 is a reasonable limit).
     htmlOptions = write_html_image_list()
+    
+    #: Whether to sort images by filename (True) or randomly (False)
     sort_by_filename = True
+    
+    #: Only show images that contain bounding boxes
     trim_to_images_with_bboxes = False
     
-    random_seed = 0 # None
+    #: Random seed to use for sampling images
+    random_seed = 0
     
-    # Should we include Web search links for each category name?    
+    #: Should we include Web search links for each category name?    
     add_search_links = False
     
-    # Should each thumbnail image link back to the original image?
+    #: Should each thumbnail image link back to the original image?
     include_image_links = False
     
-    # Should there be a text link back to each original image?
+    #: Should there be a text link back to each original image?
     include_filename_links = False
     
+    #: Line width in pixels
     box_thickness = 4
+    
+    #: Number of pixels to expand each bounding box
     box_expansion = 0
     
-    # These are mutually exclusive; both are category names, not IDs
-    classes_to_exclude = None
+    #: Only include images that contain annotations with these class names (not IDs)
+    #:
+    #: Mutually exclusive with classes_to_exclude
     classes_to_include = None
     
-    # Special tag used to say "show me all images with multiple categories"
+    #: Exclude images that contain annotations with these class names (not IDs)
+    #:
+    #: Mutually exclusive with classes_to_include
+    classes_to_exclude = None
+    
+    #: Special tag used to say "show me all images with multiple categories"
+    #:
+    #: :meta private:
     multiple_categories_tag = '*multiple*'
 
-    # We sometimes flatten image directories by replacing a path separator with 
-    # another character.  Leave blank for the typical case where this isn't necessary.
+    #: We sometimes flatten image directories by replacing a path separator with 
+    #: another character.  Leave blank for the typical case where this isn't necessary.
     pathsep_replacement = '' # '~'
 
-    # Control rendering parallelization
-    parallelize_rendering_n_cores = 25
-    
-    # Process-based parallelization in this function is currently unsupported
-    # due to pickling issues I didn't care to look at, but I'm going to just
-    # flip this with a warning, since I intend to support it in the future.
-    parallelize_rendering_with_threads = True
+    #: Parallelize rendering across multiple workers
     parallelize_rendering = False
     
-    # Should we show absolute (vs. relative) paths for each image?
+    #: In theory, whether to parallelize with threads (True) or processes (False), but
+    #: process-based parallelization in this function is currently unsupported
+    parallelize_rendering_with_threads = True
+    
+    #: Number of workers to use for parallelization; ignored if parallelize_rendering
+    #: is False
+    parallelize_rendering_n_cores = 25
+        
+    #: Should we show absolute (True) or relative (False) paths for each image?
     show_full_paths = False
     
+    #: Set to False to skip existing images
+    force_rendering = True
+    
+    #: Enable additionald debug console output
+    verbose = False
+    
 
 #%% Helper functions
 
-# Translate the file name in an image entry in the json database to a path, possibly doing
-# some manipulation of path separators
-def image_filename_to_path(image_file_name, image_base_dir, pathsep_replacement=''):
+def _image_filename_to_path(image_file_name, image_base_dir, pathsep_replacement=''):
+    """
+    Translates the file name in an image entry in the json database to a path, possibly doing
+    some manipulation of path separators.
+    """
     
     if len(pathsep_replacement) > 0:
         image_file_name = os.path.normpath(image_file_name).replace(os.pathsep,pathsep_replacement)        
@@ -96,16 +138,18 @@ def image_filename_to_path(image_file_name, image_base_dir, pathsep_replacement=
 
 #%% Core functions
 
-def process_images(db_path, output_dir, image_base_dir, options=None):
+def visualize_db(db_path, output_dir, image_base_dir, options=None):
     """
-    Writes images and html to output_dir to visualize the annotations in the json file
-    db_path.
+    Writes images and html to output_dir to visualize the annotations in a .json file.
     
-    db_path can also be a previously-loaded database.
+    Args:
+        db_path (str or dict): the .json filename to load, or a previously-loaded database
+        image_base_dir (str): the folder where the images live; filenames in [db_path] should
+            be relative to this folder.
+        options (DbVizOptions, optional): See DbVizOptions for details
     
-    Returns the html filename and the database:
-        
-    return htmlOutputFile,image_db
+    Returns:
+        tuple: A length-two tuple containing (the html filename) and (the loaded database).        
     """    
     
     if options is None:
@@ -233,7 +277,7 @@ def process_images(db_path, output_dir, image_base_dir, options=None):
             img_path = image_base_dir + img_relative_path
         else:
             img_path = os.path.join(image_base_dir, 
-                                    image_filename_to_path(img_relative_path, image_base_dir))
+                                    _image_filename_to_path(img_relative_path, image_base_dir))
     
         annos_i = df_anno.loc[df_anno['image_id'] == img_id, :] # all annotations on this image
     
@@ -264,7 +308,7 @@ def process_images(db_path, output_dir, image_base_dir, options=None):
             categoryName = label_map[categoryID]
             if options.add_search_links:
                 categoryName = categoryName.replace('"','')
-                categoryName = '<a href="https://www.bing.com/images/search?q={}">{}</a>'.format(
+                categoryName = '<a href="https://www.google.com/search?tbm=isch&q={}">{}</a>'.format(
                     categoryName,categoryName)
             imageCategories.add(categoryName)
             
@@ -309,14 +353,22 @@ def process_images(db_path, output_dir, image_base_dir, options=None):
         if options.include_filename_links:
             filename_text = '<a href="{}">{}</a>'.format(img_path,filename_text)
             
+        flagString = ''
+        
+        def isnan(x):
+            return (isinstance(x,float) and np.isnan(x))
+        
+        if ('flags' in img) and (not isnan(img['flags'])):
+            flagString = ', flags: {}'.format(str(img['flags']))
+            
         # We're adding html for an image before we render it, so it's possible this image will
         # fail to render.  For applications where this script is being used to debua a database
         # (the common case?), this is useful behavior, for other applications, this is annoying.
         image_dict = \
         {
             'filename': '{}/{}'.format('rendered_images', file_name),
-            'title': '{}<br/>{}, num boxes: {}, {}class labels: {}{}'.format(
-                filename_text, img_id, len(bboxes), frameString, imageClasses, labelLevelString),
+            'title': '{}<br/>{}, num boxes: {}, {}class labels: {}{}{}'.format(
+                filename_text, img_id, len(bboxes), frameString, imageClasses, labelLevelString, flagString),
             'textStyle': 'font-family:verdana,arial,calibri;font-size:80%;' + \
                 'text-align:left;margin-top:20;margin-bottom:5'
         }
@@ -333,7 +385,13 @@ def process_images(db_path, output_dir, image_base_dir, options=None):
         bboxes = rendering_info['bboxes']
         bboxClasses = rendering_info['boxClasses']
         output_file_name = rendering_info['output_file_name']
+        output_full_path = os.path.join(output_dir, 'rendered_images', output_file_name)
         
+        if (os.path.isfile(output_full_path)) and (not options.force_rendering):
+            if options.verbose:
+                print('Skipping existing image {}'.format(output_full_path))
+            return True
+            
         if not img_path.startswith('http'):
             if not os.path.exists(img_path):
                 print('Image {} cannot be found'.format(img_path))
@@ -348,7 +406,7 @@ def process_images(db_path, output_dir, image_base_dir, options=None):
                 image = vis_utils.resize_image(original_image, options.viz_size[0],
                                                options.viz_size[1])
         except Exception as e:
-            print('Image {} failed to open. Error: {}'.format(img_path, e))
+            print('Image {} failed to open, error: {}'.format(img_path, e))
             return False
             
         vis_utils.render_db_bounding_boxes(boxes=bboxes, classes=bboxClasses,
@@ -357,7 +415,8 @@ def process_images(db_path, output_dir, image_base_dir, options=None):
                                            thickness=options.box_thickness,
                                            expansion=options.box_expansion)
         
-        image.save(os.path.join(output_dir, 'rendered_images', output_file_name))
+        image.save(output_full_path)
+                
         return True
     
     # ...def render_image_info
@@ -400,6 +459,8 @@ def process_images(db_path, output_dir, image_base_dir, options=None):
         
     if options.sort_by_filename:    
         images_html = sorted(images_html, key=lambda x: x['filename'])
+    else:
+        random.shuffle(images_html)
         
     htmlOutputFile = os.path.join(output_dir, 'index.html')
     
@@ -418,7 +479,7 @@ def process_images(db_path, output_dir, image_base_dir, options=None):
     
     return htmlOutputFile,image_db
 
-# def process_images(...)
+# def visualize_db(...)
     
     
 #%% Command-line driver
@@ -452,7 +513,8 @@ def main():
     parser.add_argument('--random_seed', action='store', type=int, default=None,
                         help='Random seed for image selection')
     parser.add_argument('--pathsep_replacement', action='store', type=str, default='',
-                        help='Replace path separators in relative filenames with another character (frequently ~)')
+                        help='Replace path separators in relative filenames with another ' + \
+                             'character (frequently ~)')
     
     if len(sys.argv[1:]) == 0:
         parser.print_help()
@@ -466,15 +528,13 @@ def main():
     if options.random_sort:
         options.sort_by_filename = False
         
-    process_images(options.db_path,options.output_dir,options.image_base_dir,options) 
+    visualize_db(options.db_path,options.output_dir,options.image_base_dir,options) 
 
-
-if __name__ == '__main__':
-    
+if __name__ == '__main__':    
     main()
 
 
-#%% Interactive driver(s)
+#%% Interactive driver
 
 if False:
     
@@ -487,6 +547,5 @@ if False:
     options = DbVizOptions()
     options.num_to_visualize = 100
     
-    htmlOutputFile,db = process_images(db_path,output_dir,image_base_dir,options)
+    htmlOutputFile,db = visualize_db(db_path,output_dir,image_base_dir,options)
     # os.startfile(htmlOutputFile)
-
